@@ -88,21 +88,68 @@ PROPERTY_LINE_REGEX_VALUE = r"^[ ]*([\w\-]*)[ ]*:[ ]*(\S*)"
 PROPERTY_LINE_REGEX = re.compile(PROPERTY_LINE_REGEX_VALUE)
 """ The property line regex """
 
-PROPERTY_LINE_REPLACEMENT_VALUE = "    \g<1>: \g<2>"
+PROPERTY_LINE_REPLACEMENT_VALUE = "\g<1>: \g<2>"
 """ The property line replacement value """
 
-RULES_MAP = dict(
-    left = ("zero_to_zero_px",),
-    top = ("zero_to_zero_px",),
-    right = ("zero_to_zero_px",),
-    bottom = ("zero_to_zero_px",),
-    border = ("zero_to_none",),
-    margin = ("zero_to_multiple", "two_to_multiple"),
-    padding = ("zero_to_multiple", "two_to_multiple")
-)
+TOKEN_REGEX_VALUE = r"\s*([>\+\s])\s*"
+""" The regex value meant to catch the various css related
+tokens that are separated with a space compatible character """
+
+TOKEN_REGEX = re.compile(TOKEN_REGEX_VALUE)
+""" The regex meant to catch the various css related
+tokens that are separated with a space compatible character """
+
+COMMA_REGEX_VALUE = r"\s*(,)\s*"
+""" The regex value that defines a valid comma under css,
+note that multiple characters before and after are allowed """
+
+COMMA_REGEX = re.compile(COMMA_REGEX_VALUE)
+""" The regex that defines a valid comma under css,
+note that multiple characters before and after are allowed """
+
+SPACE_REGEX_VALUE = r"\s+"
+""" The regex value that defines a valid space under css,
+note that multiple characters are allowed """
+
+SPACE_REGEX = re.compile(SPACE_REGEX_VALUE)
+""" The regex that defines a valid space under css,
+note that multiple characters are allowed """
+
+RULES_MAP = {
+    "left" : ("zero_to_zero_px",),
+    "top" : ("zero_to_zero_px",),
+    "right" : ("zero_to_zero_px",),
+    "bottom" : ("zero_to_zero_px",),
+    "border" : ("zero_to_none",),
+    "margin" : (
+        "zero_to_multiple",
+        "one_to_multiple",
+        "two_to_multiple"
+    ),
+    "padding" : (
+        "zero_to_multiple",
+        "one_to_multiple",
+        "two_to_multiple"
+    ),
+    "border-radius" : (
+        "zero_to_multiple",
+        "one_to_multiple",
+        "two_to_multiple"
+    )
+}
 """ The map that associates various property names
 with the various rules that should be applied to them """
 
+BROWSER_PREFIXES = (
+    "-o",
+    "-ms",
+    "-moz",
+    "-khtml",
+    "-webkit"
+)
+""" The sequence that defines the complete set of accepted
+browser prefixes for the stylesheet usage """
+
 def get_property_index(property_line, property_order, line_number):
     """
     Retrieves the index for the property specified in the provided property
@@ -114,6 +161,9 @@ def get_property_index(property_line, property_order, line_number):
     @param property_order: The list of property names in order.
     @type line_number: int
     @param line_number: The approximate line number for this processing.
+    @rtype: int
+    @return: The index of the property line, note that in case an index
+    value is not found for the rule a large index is returned instead.
     """
 
     # splits the property name, to retrieve the property value
@@ -145,13 +195,12 @@ def get_property_index(property_line, property_order, line_number):
 
     # in case the property is not in the order
     if not property_name in property_order:
-        # warns about the missing property name
+        # warns about the missing property name and returns the
+        # largest possible index value for any property name
         extra.warn(
             "Order for property %s not defined at line %d" %\
             (property_name, line_number)
         )
-
-        # uses the greatest index
         return len(property_order)
 
     # determines the index for the property name and returns
@@ -191,8 +240,8 @@ def write_line(output_buffer, line, windows_newline = True, avoid_empty = False)
 
     @type output_buffer: StringBuffer
     @param output_buffer: The output buffer.
-    @type lines: List
-    @param lines: The list of lines to output.
+    @type line: List
+    @param line: The line that is going to be written to output buffer.
     @type windows_newline: bool
     @param windows_newline: If the windows newline should be used.
     @type avoid_empty: bool
@@ -216,7 +265,12 @@ def write_line(output_buffer, line, windows_newline = True, avoid_empty = False)
     if windows_newline: output_buffer.write("\r\n")
     else: output_buffer.write("\n")
 
-def process_property_lines(property_lines, line_number, avoid_empty = False):
+def process_property_lines(
+    property_lines,
+    line_number,
+    indentation = 0,
+    avoid_empty = False
+):
     """
     Processes the property lines, this process should be
     iterative for each of the lines passed in the list.
@@ -228,6 +282,10 @@ def process_property_lines(property_lines, line_number, avoid_empty = False):
     @type avoid_empty: bool
     @param avoid_empty: If the empty lines should avoid any
     kind of processing (extra errors may occur).
+    @type indentation: int
+    @param identation: The level of indentation to be used for the processing
+    of the various property lines, this value is going to be used in the
+    operation of pre-pending spaced to the property rules.
     @rtype: List
     @return: The processed property lines.
     """
@@ -235,13 +293,13 @@ def process_property_lines(property_lines, line_number, avoid_empty = False):
     # processes the property lines one by one so that the
     # properly represent the same semantic value but with
     # a better/standard representation of the value
-    processed_property_lines = [process_property_line(property_line, line_number)\
+    processed_property_lines = [process_property_line(property_line, line_number, indentation)\
         for property_line in property_lines if not avoid_empty or property_line.strip()]
 
     # returns the processed property lines
     return processed_property_lines
 
-def process_property_line(property_line, line_number):
+def process_property_line(property_line, line_number, indentation):
     # in case the property line is empty, when stripped the property
     # is considered to be empty (warning required) logs the message
     # and then returns the line itself (no processing)
@@ -260,7 +318,11 @@ def process_property_line(property_line, line_number):
 
     # ensures the property line is correctly indented and
     # the property name and value are correctly separated
-    property_line = PROPERTY_LINE_REGEX.sub(PROPERTY_LINE_REPLACEMENT_VALUE, property_line)
+    padding = indentation * " " * 4
+    property_line = PROPERTY_LINE_REGEX.sub(
+        padding + PROPERTY_LINE_REPLACEMENT_VALUE,
+        property_line
+    )
 
     # ensures the property line and an ending semicolon
     # adding it to the property line in case it does not
@@ -291,6 +353,11 @@ def rule_zero_to_none(name, value):
     if not value in ("0", "0px"): return
     return "%s: none;" % name
 
+def rule_one_to_multiple(name, value):
+    parts = value.split()
+    if not len(parts) == 1: return
+    return "%s: %s %s %s %s;" % (name, value, value, value, value)
+
 def rule_two_to_multiple(name, value):
     parts = value.split()
     if not len(parts) == 2: return
@@ -303,10 +370,18 @@ def process_rules(property_line, line_number):
     property_name, _property_value = property_line.split(":", 1)
     property_name = property_name.strip()
 
+    # constructs the base name, by removing the complete set
+    # of browser prefixes, this value is going to be used as
+    # the basis for the retrieval of rules (wildcard retrieval)
+    base_name = property_name
+    for prefix in BROWSER_PREFIXES:
+        if not base_name.startswith(prefix): continue
+        base_name = base_name[len(prefix) + 1:]
+
     # tries to retrieve the set of rules (as strings) that are
     # going to be applied to the current property and then runs
     # the iteration cycle to apply each of them on the line
-    rules = RULES_MAP.get(property_name, [])
+    rules = RULES_MAP.get(base_name, [])
     for rule in rules:
         # creates the (rule) method name by appending the name of
         # the current rule to the prefix and tries to retrieve a
@@ -461,7 +536,11 @@ def cleanup_properties(
         line_number += 1
 
         # updates the comparison key function
-        get_comparison_key = lambda property_line: get_property_index(property_line, property_order, line_number)
+        get_comparison_key = lambda property_line: get_property_index(
+            property_line,
+            property_order,
+            line_number
+        )
 
         # in case the line contains a single line comment
         if "/*" in line and "*/" in line:
@@ -482,10 +561,9 @@ def cleanup_properties(
                 "Found closing comment without corresponding opening at line %d" % line_number
             )
 
-            # decrements the comments started counter
-            comments_started -= 1
-
+            # decrements the comments started counter and then
             # enables the needs newline flag
+            comments_started -= 1
             needs_newline = True
         # in case this is a comment line
         elif comments_started:
@@ -496,61 +574,98 @@ def cleanup_properties(
             # after an at rule, a newline must follow
             needs_newline = True
         # in case the line contains a full rule specification
+        # does nothing, will just write line as is as there's
+        # very few thing possible to optimise under this situation
         elif "{" in line and "}" in line:
-            # does nothing, will just write line as is
+            line = line.strip()
             needs_newline = True
+            rule_started = False
         # in case this is a rule start line
         elif "{" in line:
             # increments the open rule count
             open_rule_count += 1
 
-            # in case this is an actual rule
-            if open_rule_count == 1:
+            # in case this is an actual rule, meaning that the open rule
+            # count value is positive (assertion valid)
+            if open_rule_count > 0:
                 # resets the rule lines
                 rule_lines = []
 
                 # signals the rule started flag,
                 # in case the rule is no to be skipped
                 rule_started = not skip_rule(line, rules_skip)
+
+            # replaces the various css tokens with proper separators
+            # meaning that no extra spaces between them are allowed
+            line = TOKEN_REGEX.sub(r" \1 ", line)
+
+            # runs the space replacement operation so that the
+            # extra spaces in the line (not required) are removed
+            # and the line is simplified
+            line = SPACE_REGEX.sub(" ", line)
+            line = line.strip()
+
+            # calculates the space padding that is going to be pre-pended
+            # to the line and then adds it to the line note that the open
+            # rule count is decremented by one (opened on this line)
+            padding = (open_rule_count - 1) * 4 * " "
+            line = padding + line
+
+            # flushes the current line value to the output buffer and then
+            # continues the current loop (avoids general operations)
+            write_line(output_buffer, line, windows_newline)
+            continue
+
         elif "}" in line:
             # decrements the open rule count
             open_rule_count -= 1
 
-            # in case this is an actual rule
-            if open_rule_count == 0:
-                # in case the rule set does not contain any property
-                # must log an information message about the empty rule
-                if rule_started and not rule_lines:
-                    extra.warn("Empty stylesheet rule at line %d" % line_number)
+            # in case there is a mismatch in open and closed rules
+            # must raise an exception indicating the problem
+            if open_rule_count < 0: raise Exception("mismatched rules found")
 
-                # updates the flag to signal the rule has ended
-                rule_started = False
+            # strips the current line removing any extra (not expected)
+            # values and simplifying the current line value
+            padding = open_rule_count * 4 * " "
+            line = line.strip()
+            line = padding + line
 
-                # sorts the various property lines and the processes them
-                property_lines = sorted(rule_lines, key = get_comparison_key)
-                property_lines = process_property_lines(
-                    property_lines,
-                    line_number,
-                    avoid_empty = fix_extra_newlines
-                )
+            # in case the rule set does not contain any property
+            # must log an information message about the empty rule
+            if rule_started and not rule_lines:
+                extra.warn("Empty stylesheet rule at line %d" % line_number)
 
-                # writes the lines to the buffer, considering the windows newline
-                # and then writes the line
-                write_lines(
-                    output_buffer,
-                    property_lines,
-                    windows_newline = windows_newline,
-                    avoid_empty = fix_extra_newlines
-                )
-                write_line(output_buffer, line, windows_newline = windows_newline)
+            # updates the flag to signal the rule has ended
+            rule_started = False
 
-                # resets the newlines counter and then
-                # enables the needs newline flag
-                newlines = 0
-                needs_newline = True
+            # sorts the various property lines and the processes them
+            property_lines = sorted(rule_lines, key = get_comparison_key)
+            property_lines = process_property_lines(
+                property_lines,
+                line_number,
+                indentation = open_rule_count + 1,
+                avoid_empty = fix_extra_newlines
+            )
+
+            # writes the lines to the buffer, considering the windows newline
+            # and then writes the line
+            write_lines(
+                output_buffer,
+                property_lines,
+                windows_newline = windows_newline,
+                avoid_empty = fix_extra_newlines
+            )
+            write_line(output_buffer, line, windows_newline = windows_newline)
+
+            # resets the newlines counter and then
+            # enables the needs newline flag
+            newlines = 0
+            needs_newline = True
+            rule_lines = []
+
+            # skips further processing
+            continue
 
-                # skips further processing
-                continue
         # in case this line is part of a valid rule set
         elif rule_started:
             # appends the line to the rule set, and then
@@ -558,9 +673,12 @@ def cleanup_properties(
             rule_lines.append(line)
             continue
         # in case this is part of rule selector declaration
+        # the typical rule replacement operations are applied
         elif "," in line:
-            # does nothing, will just write line as is
-            pass
+            line = TOKEN_REGEX.sub(r" \1 ", line)
+            line = SPACE_REGEX.sub(" ", line)
+            line = COMMA_REGEX.sub(r"\1", line)
+            line = line.strip()
         # in case the between rules mode is active
         elif not rule_started and not line.strip():
             # increments the newlines count
@@ -578,19 +696,24 @@ def cleanup_properties(
             # warns about the statement outside a valid rule
             extra.warn("Found statement outside rule at line %d" % line_number)
 
+        # "calculates" the padding string value taking into account the
+        # current open rule count value and then removes any left space
+        # compatible values and replaces them with proper indentation
+        padding = open_rule_count * 4 * " "
+        line = line.lstrip()
+        line = padding + line
+
         # writes the line to the output buffer taking into
         # account the windows newline control flag
         write_line(output_buffer, line, windows_newline)
 
     # in case there is a mismatch in open and closed rules
     # must raise an exception indicating the problem
-    if not open_rule_count == 0:
-        raise Exception("mismatched rules found")
+    if not open_rule_count == 0: raise Exception("mismatched rules found")
 
-    # retrieves the output buffer value
+    # retrieves the output buffer value and then counts the
+    # number of lines contained in it (assertion validation)
     output_buffer_value = output_buffer.getvalue()
-
-    # counts the lines in the output buffer
     number_lines = output_buffer_value.count("\n")
 
     if not number_lines == number_original_lines and not fix_extra_newlines:
