@@ -54,9 +54,11 @@ configurations = (
             "cpp",
             "c",
             "m",
+            "mm",
             "hpp",
             "h",
             "pch",
+            "lua",
             "cs",
             "st",
             "ctp",
